@@ -69,9 +69,9 @@ def show_banner():
     click.secho(f"                        v{VERSION}", fg="bright_white", bold=True)
     click.secho("=" * 60, fg="bright_blue", bold=True)
 
-def update_current_schema(client, db_name, pipeline_id, version=None):
-    """Update the current_schema.json with the latest or specified schema version."""
-    schema = load_schema(client, db_name, pipeline_id, version)
+def update_current_schema(client, db_name, pipeline_id):
+    """Update the current_schema.json with the latest schema."""
+    schema = load_schema(client, db_name, pipeline_id)
     if schema:
         schema_dir = Path(f"schemas/{pipeline_id}")
         schema_dir.mkdir(parents=True, exist_ok=True)
@@ -79,7 +79,6 @@ def update_current_schema(client, db_name, pipeline_id, version=None):
         meta = {
             "_meta": {
                 "pipeline_id": pipeline_id,
-                "version": schema.get("schema_version", VERSION),
                 "timestamp": datetime.utcnow().isoformat() + "Z"
             },
             "schema": schema
@@ -88,7 +87,7 @@ def update_current_schema(client, db_name, pipeline_id, version=None):
             with current_schema_path.open("w") as f:
                 json.dump(meta, f, indent=2)
             current_schema_path.chmod(stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
-            click.secho(f"✓ Updated current_schema.json for {pipeline_id} (version: {schema.get('schema_version', VERSION)})", fg="bright_green", bold=True)
+            click.secho(f"✓ Updated current_schema.json for {pipeline_id}", fg="bright_green", bold=True)
         except PermissionError:
             click.secho(f"✗ Failed to update current_schema.json for {pipeline_id} due to permissions. Run 'chmod u+w {current_schema_path}' manually.", fg="red", bold=True)
     else:
@@ -145,13 +144,13 @@ class ColoredGroup(click.Group):
         click.secho("\n  Initialization:", fg="bright_green", bold=True)
         click.secho("    init          Initialize a new main pipeline with a JSON schema and MongoDB setup.", fg="bright_cyan")
         click.secho("    sub-init      Initialize a sub-schema inheriting from main schema.", fg="bright_cyan")
-        click.secho("    new-version   Create a new schema version based on semantic versioning (local copy only).", fg="bright_cyan")
+        click.secho("    new-version   Create a new schema version (local copy only).", fg="bright_cyan")
 
         click.secho("\n  Schema Management:", fg="bright_green", bold=True)
         click.secho("    schema-compile  Validate a schema file.", fg="bright_cyan")
-        click.secho("    schema-push     Push a schema to MongoDB with versioning.", fg="bright_cyan")
+        click.secho("    schema-push     Push a schema to MongoDB.", fg="bright_cyan")
         click.secho("    schema-pull     Pull a schema from MongoDB.", fg="bright_cyan")
-        click.secho("    schema-current  Display the current schema version.", fg="bright_cyan")
+        click.secho("    schema-current  Display the current schema.", fg="bright_cyan")
         click.secho("    version-list    List schema versions for a pipeline.", fg="bright_cyan")
         click.secho("    version-rollback  Rollback to a previous schema version.", fg="bright_cyan")
 
@@ -470,7 +469,6 @@ def init(file, interactive, backend_host, frontend_host):
 
     schema = {
         "pipeline_id": pipeline_id,
-        "schema_version": VERSION,
         "description": description,
         "created_by": created_by,
         "created_at": datetime.utcnow().isoformat() + "Z",
@@ -594,7 +592,6 @@ def sub_init(main_schema, interactive):
 
     schema = {
         "pipeline_id": pipeline_id,
-        "schema_version": VERSION,
         "description": description,
         "created_by": created_by,
         "created_at": datetime.utcnow().isoformat() + "Z",
@@ -648,7 +645,7 @@ def sub_init(main_schema, interactive):
 @cli.command()
 @click.option('--pipeline-id', required=True, help='Pipeline ID')
 def new_version(pipeline_id):
-    """Create a new schema version based on semantic versioning (local copy only)."""
+    """Create a new schema version (local copy only)."""
     show_banner()
     click.secho(f"\nCreating New Version for Pipeline {pipeline_id}...", fg="bright_yellow", bold=True)
     try:
@@ -663,48 +660,24 @@ def new_version(pipeline_id):
         versions = list_versions(client, db_name, pipeline_id)
         
         if not versions:
-            click.secho(f"✗ No versions found for {pipeline_id}. Starting with v0.1.0.", fg="red", bold=True)
+            click.secho(f"✗ No versions found for {pipeline_id}. Starting with new version.", fg="red", bold=True)
             latest_version = "v0.0.0"
         else:
-            latest_version = max(versions, key=lambda x: [int(i) for i in x['schema_version'].replace('v', '').split('.')])['schema_version']
+            latest_version = "v0.0.0"  # Default for naming, since schema_version is removed
 
-        click.secho(f"Current version: {latest_version}", fg="bright_blue")
-        bump_type = click.prompt(
-            click.style("Select version bump:", fg="bright_blue"),
-            type=click.Choice(["1", "2", "3"]),
-            show_choices=True,
-            prompt_suffix="\n1. patch   - Fixes or small changes (e.g. v0.4.3)\n"
-                         "2. minor   - Add new task(s) or non-breaking config changes (e.g. v0.5.0)\n"
-                         "3. major   - Breaking changes, redesign, removal (e.g. v1.0.0)\n> "
-        )
+        click.secho(f"Creating new version for {pipeline_id}", fg="bright_blue")
+        new_version = f"v{datetime.utcnow().strftime('%Y%m%d_%H%M%S')}"  # Use timestamp for versioning
         
-        latest_nums = [int(i) for i in latest_version.replace('v', '').split('.')]
-        if bump_type == "1":  # patch
-            latest_nums[2] += 1
-        elif bump_type == "2":  # minor
-            latest_nums[1] += 1
-            latest_nums[2] = 0
-        else:  # major
-            latest_nums[0] += 1
-            latest_nums[1] = 0
-            latest_nums[2] = 0
-            breaking_changes = click.prompt(click.style("What breaking changes are you introducing? (description)", fg="bright_blue"), default="")
-
-        new_version = f"v{latest_nums[0]}.{latest_nums[1]}.{latest_nums[2]}"
-        
-        schema = load_schema(client, db_name, pipeline_id, latest_version)
+        schema = load_schema(client, db_name, pipeline_id)
         if not schema:
-            click.secho(f"✗ No schema found for version {latest_version}.", fg="red", bold=True)
+            click.secho(f"✗ No schema found for {pipeline_id}.", fg="red", bold=True)
             return
         
-        schema["schema_version"] = new_version
         schema["created_at"] = datetime.utcnow().isoformat() + "Z"
         schema["created_by"] = os.getenv("USER", "unknown")
         schema["metadata"]["parent_version"] = latest_version
         schema["metadata"]["status"] = "draft"
         schema["metadata"]["changelog"] = [f"Auto-generated new version {new_version}"]
-        if bump_type == "3" and breaking_changes:
-            schema["metadata"]["changelog"].append(f"Breaking changes: {breaking_changes}")
 
         schema_dir = Path(f"schemas/{pipeline_id}")
         schema_dir.mkdir(parents=True, exist_ok=True)
@@ -749,7 +722,7 @@ def schema_compile(main_schema):
 @cli.command()
 @click.option('--main-schema', type=click.Path(exists=True), required=True, help='Path to main schema file (e.g., schemas/production.json)')
 def schema_push(main_schema):
-    """Push a main schema and its sub-schemas to MongoDB with versioning."""
+    """Push a main schema and its sub-schemas to MongoDB."""
     click.secho("\nPushing Schema to MongoDB...", fg="bright_yellow", bold=True)
     try:
         config_path = Path("config/default_config.yaml")
@@ -760,18 +733,18 @@ def schema_push(main_schema):
         schema = load_main_schema(main_schema)
         validate_schema(schema)
         push_schema(client, db_name, schema)
-        click.secho(f"✓ Main schema {schema['pipeline_id']} pushed to MongoDB with version {schema['schema_version']}.", fg="bright_green", bold=True)
+        click.secho(f"✓ Main schema {schema['pipeline_id']} pushed to MongoDB.", fg="bright_green", bold=True)
         update_current_schema(client, db_name, schema["pipeline_id"])
-        copy_schema_file(schema["pipeline_id"], schema["metadata"].get("parent_version", "v0.0.0"), schema["schema_version"])
+        copy_schema_file(schema["pipeline_id"], schema["metadata"].get("parent_version", "v0.0.0"), datetime.utcnow().strftime("%Y%m%d_%H%M%S"))
         for sub_schema_path in schema["metadata"]["sub_schemas"]:
             try:
                 with open(sub_schema_path, 'r') as f:
                     sub_schema = json.load(f)
                 validate_schema(sub_schema)
                 push_schema(client, db_name, sub_schema)
-                click.secho(f"✓ Sub-schema {sub_schema['pipeline_id']} pushed to MongoDB with version {sub_schema['schema_version']}.", fg="bright_green", bold=True)
+                click.secho(f"✓ Sub-schema {sub_schema['pipeline_id']} pushed to MongoDB.", fg="bright_green", bold=True)
                 update_current_schema(client, db_name, sub_schema["pipeline_id"])
-                copy_schema_file(sub_schema["pipeline_id"], sub_schema["metadata"].get("parent_version", "v0.0.0"), sub_schema["schema_version"])
+                copy_schema_file(sub_schema["pipeline_id"], sub_schema["metadata"].get("parent_version", "v0.0.0"), datetime.utcnow().strftime("%Y%m%d_%H%M%S"))
             except ValueError as e:
                 click.secho(f"✗ Validation error in sub-schema '{sub_schema['pipeline_id']}' ({sub_schema_path}): {e}", fg="red", bold=True)
                 sys.exit(1)
@@ -869,7 +842,7 @@ def version_list(pipeline_id):
         click.secho(f"Found {len(versions)} version(s):", fg="bright_green")
         for v in versions:
             is_latest = " (latest)" if v.get("is_latest", False) else ""
-            click.secho(f"  • Version {v['schema_version']}{is_latest}: Created {v['created_at']}", fg="bright_cyan")
+            click.secho(f"  • Version {v.get('schema_version', 'unknown')}{is_latest}: Created {v['created_at']}", fg="bright_cyan")
     except Exception as e:
         click.secho(f"✗ Error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
@@ -888,7 +861,7 @@ def version_rollback(pipeline_id, version):
         db_name = config["db_name"]
         rollback_version(client, db_name, pipeline_id, version)
         click.secho(f"✓ Rolled back {pipeline_id} to version {version}.", fg="bright_green", bold=True)
-        update_current_schema(client, db_name, pipeline_id, version)
+        update_current_schema(client, db_name, pipeline_id)
     except ValueError as e:
         click.secho(f"✗ Rollback error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
@@ -1155,7 +1128,7 @@ def evaluate(task_id):
 @cli.command()
 @click.option('--pipeline_id', required=True, help='Pipeline ID')
 def schema_current(pipeline_id):
-    """Display the current schema version."""
+    """Display the current schema."""
     click.secho(f"\nDisplaying Current Schema for Pipeline {pipeline_id}...", fg="bright_yellow", bold=True)
     try:
         config_path = Path("config/default_config.yaml")
@@ -1165,7 +1138,6 @@ def schema_current(pipeline_id):
         db_name = config["db_name"]
         schema = load_schema(client, db_name, pipeline_id)
         if schema:
-            click.secho(f"Current version: {schema['schema_version']}", fg="bright_green")
             click.secho(f"Created at: {schema['created_at']}", fg="bright_cyan")
             click.secho(f"Description: {schema['description']}", fg="bright_cyan")
         else:
