@@ -4,6 +4,7 @@ from sentence_transformers import SentenceTransformer
 import logging
 from typing import List, Dict, Optional
 import time
+import yaml
 
 # Configure logging
 logging.basicConfig(level=logging.INFO)
@@ -15,13 +16,23 @@ _db_client = None
 class ChromaClient:
     """A client for managing ChromaDB vector database operations."""
     
-    def __init__(self, db_path: str = "./contextchain_chromadb"):
+    def __init__(self, db_path: str = None):
         """
         Initialize the ChromaDB client with a persistent storage path.
 
         Args:
-            db_path (str): The file system path to store the vector database.
+            db_path (str, optional): The file system path to store the vector database.
+                                    If None, reads from config.yaml.
         """
+        if db_path is None:
+            config_path = Path("config/default_config.yaml")
+            if config_path.exists():
+                with config_path.open("r") as f:
+                    config = yaml.safe_load(f)
+                db_path = config.get("chroma_path", "./contextchain_chromadb")
+            else:
+                db_path = "./contextchain_chromadb"
+        
         self.db_path = Path(db_path)
         self.db_path.mkdir(parents=True, exist_ok=True)
         self.client = chromadb.PersistentClient(path=str(self.db_path))
@@ -69,12 +80,11 @@ class ChromaClient:
             Dict: Metrics (e.g., {'added': count, 'time_taken': seconds}).
         """
         try:
-            collection = self.client.get_collection(name=collection_name)
+            collection = self.client.get_or_create_collection(name=collection_name)
             start_time = time.time()
-            ids = [f"doc_{i}" for i in range(len(documents))]
+            ids = [f"doc_{i}_{int(start_time)}" for i in range(len(documents))]
             embeddings = self.embedding_model.encode(documents, batch_size=batch_size).tolist()
             
-            # Process in batches to handle large data
             for i in range(0, len(documents), batch_size):
                 batch_ids = ids[i:i + batch_size]
                 batch_docs = documents[i:i + batch_size]
@@ -110,7 +120,7 @@ class ChromaClient:
             Dict: Search results and metrics (e.g., {'results': [...], 'time_taken': seconds}).
         """
         try:
-            collection = self.client.get_collection(name=collection_name)
+            collection = self.client.get_or_create_collection(name=collection_name)
             start_time = time.time()
             query_embedding = self.embedding_model.encode([query])[0].tolist()
             results = collection.query(
@@ -127,18 +137,19 @@ class ChromaClient:
             logger.error(f"✗ Failed to search in {collection_name}: {e}")
             raise
 
-def get_vector_db_client(db_path: str = "./contextchain_chromadb") -> ChromaClient:
+def get_vector_db_client(db_path: str = None) -> ChromaClient:
     """
     Get or initialize a singleton ChromaDB client.
 
     Args:
-        db_path (str): The file system path to store the vector database.
+        db_path (str, optional): The file system path to store the vector database.
+                                If None, reads from config.yaml.
 
     Returns:
         ChromaClient: An initialized and connected ChromaDB client instance.
     """
     global _db_client
-    if _db_client:
+    if _db_client and (db_path is None or _db_client.db_path == Path(db_path)):
         return _db_client
 
     try:
