@@ -21,6 +21,7 @@ import concurrent.futures
 import networkx as nx
 from pathlib import Path
 import yaml
+import jsonschema  # Added for JSON schema validation
 
 # Set up logging
 logging.basicConfig(level=logging.INFO)
@@ -67,7 +68,7 @@ def resolve_dependencies(tasks: List[Dict[str, Any]], task_id: int, context: Dic
 
     for input_id in inputs:
         if isinstance(input_id, int) and input_id not in context:
-            raise ValueError(f"Dependency {input_id} not executed before task {task_id}")
+            raise ValueError(f"Dependency {input_id} not executed before task {task_id} in pipeline {task.get('pipeline_id', 'unknown')}")
         resolved_context[input_id] = context.get(input_id, {})
 
     payload = {}
@@ -90,7 +91,7 @@ def execute_http_request(url: str, method: str, payload: Dict[str, Any], headers
             response.raise_for_status()
             return {"output": response.json(), "status": "success"}
         except requests.RequestException as e:
-            logger.error(f"HTTP request failed (attempt {attempt + 1}/{retries + 1}): {str(e)}")
+            logger.error(f"HTTP request failed (attempt {attempt + 1}/{retries + 1}) for {url}: {str(e)}")
             if attempt == retries:
                 return {"status": "failed", "error": str(e)}
             time.sleep(2 ** attempt)
@@ -136,6 +137,8 @@ def execute_llm_request(llm_config: Dict[str, Any], prompt: str, task_model: str
 
 def execute_task(task: Dict[str, Any], schema: Dict[str, Any], db: Any, sub_schemas: List[Dict[str, Any]] = None) -> Dict[str, Any]:
     """Execute a single task based on its type."""
+    pipeline_id = schema.get("pipeline_id", "unknown")
+    task_id = task.get("task_id", "unknown")
     output_collection = task.get("output_collection", "task_results")
     result = {}
     parameters = task.get("parameters", {})
@@ -145,6 +148,25 @@ def execute_task(task: Dict[str, Any], schema: Dict[str, Any], db: Any, sub_sche
         if task["task_type"] == "LLM":
             prompt = task.get("prompt_template", "").format(**parameters)
             result = execute_llm_request(schema["global_config"]["llm_config"], prompt, task.get("model"))
+        elif task["task_type"] == "LLM_GENERATE":
+            llm_client = OllamaClient(model=schema["global_config"].get("llm_config", {}).get("model", "mistral:7b"))
+            prompt = parameters.get("prompt", "")
+            max_tokens = parameters.get("max_tokens", 512)
+            response = llm_client.generate(prompt, max_tokens)
+            if parameters.get("output_format") == "json":
+                try:
+                    json_response = json.loads(response)
+                    if "json_schema" in parameters:
+                        jsonschema.validate(json_response, parameters["json_schema"])
+                    result = {"output": json_response, "status": "success"}
+                except json.JSONDecodeError as e:
+                    logger.error(f"JSON parsing failed for task {task_id} in pipeline {pipeline_id}: {str(e)}")
+                    return {"status": "failed", "error": f"Invalid JSON output: {str(e)}", "task_id": task_id}
+                except jsonschema.ValidationError as e:
+                    logger.error(f"JSON schema validation failed for task {task_id} in pipeline {pipeline_id}: {str(e)}")
+                    return {"status": "failed", "error": f"JSON schema validation failed: {str(e)}", "task_id": task_id}
+            else:
+                result = {"output": {"response": response}, "status": "success"}
         elif task["task_type"] == "LOCAL":
             module, func = task["endpoint"].rsplit(".", 1)
             mod = importlib.import_module(module)
@@ -163,13 +185,13 @@ def execute_task(task: Dict[str, Any], schema: Dict[str, Any], db: Any, sub_sche
             metadata = parameters.get("metadata", None)
             collection_name = parameters.get("collection_name", "default_collection")
             if not vdb_client.heartbeat():
-                raise ValueError("ChromaDB client is not operational")
-            vdb_client.create_collection(collection_name)  # Ensure collection exists
+                raise ValueError(f"ChromaDB client is not operational for task {task_id} in pipeline {pipeline_id}")
+            vdb_client.create_collection(collection_name)
             metrics = vdb_client.add_documents(collection_name, documents, metadata)
             result = {"output": metrics, "status": "success"}
             db["metrics"].insert_one({
-                "pipeline_id": schema["pipeline_id"],
-                "task_id": task["task_id"],
+                "pipeline_id": pipeline_id,
+                "task_id": task_id,
                 "metrics": metrics,
                 "timestamp": datetime.utcnow().isoformat() + "Z"
             })
@@ -178,13 +200,13 @@ def execute_task(task: Dict[str, Any], schema: Dict[str, Any], db: Any, sub_sche
             k = parameters.get("k", 5)
             collection_name = parameters.get("collection_name", "default_collection")
             if not vdb_client.heartbeat():
-                raise ValueError("ChromaDB client is not operational")
-            vdb_client.create_collection(collection_name)  # Ensure collection exists
+                raise ValueError(f"ChromaDB client is not operational for task {task_id} in pipeline {pipeline_id}")
+            vdb_client.create_collection(collection_name)
             metrics = vdb_client.search(collection_name, query, k)
             result = {"output": metrics["results"], "status": "success"}
             db["metrics"].insert_one({
-                "pipeline_id": schema["pipeline_id"],
-                "task_id": task["task_id"],
+                "pipeline_id": pipeline_id,
+                "task_id": task_id,
                 "metrics": {"time_taken": metrics["time_taken"]},
                 "timestamp": datetime.utcnow().isoformat() + "Z"
             })
@@ -199,45 +221,39 @@ def execute_task(task: Dict[str, Any], schema: Dict[str, Any], db: Any, sub_sche
             summary = summarize_text(text, model)
             result = {"output": {"summary": summary}, "status": "success"}
         elif task["task_type"] == "EVALUATE":
-            results = list(db["task_results"].find({"task_id": task["task_id"]}))
+            results = list(db["task_results"].find({"task_id": task_id}))
             evaluation = evaluate_results(results)
             result = {"output": {"evaluation": evaluation}, "status": "success"}
-        elif task["task_type"] == "LLM_GENERATE":
-            llm_client = OllamaClient(model=schema["global_config"].get("llm_config", {}).get("model", "mistral:7b"))
-            prompt = parameters.get("prompt", "")
-            max_tokens = parameters.get("max_tokens", 512)
-            response = llm_client.generate(prompt, max_tokens)
-            result = {"output": {"response": response}, "status": "success"}
         else:
             handler = get_task_handler(task["task_type"])
             if handler:
                 inputs = fetch_inputs(task, db, schema, sub_schemas)
                 result = handler(task, schema, inputs)
             else:
-                raise ValueError(f"Unknown task type: {task['task_type']}")
+                raise ValueError(f"Unknown task type {task['task_type']} for task {task_id} in pipeline {pipeline_id}")
 
         # Handle rerun logic
         if task.get("rerun", False):
             existing_result = db[output_collection].find_one({
-                "pipeline_id": schema["pipeline_id"],
+                "pipeline_id": pipeline_id,
                 "schema_version": schema["schema_version"],
-                "task_id": task["task_id"]
+                "task_id": task_id
             })
             previous_data = existing_result.get("output", {}) if existing_result else {}
 
             if isinstance(result, dict):
                 updated_result = result.copy()
                 updated_result.update({
-                    "pipeline_id": schema["pipeline_id"],
+                    "pipeline_id": pipeline_id,
                     "schema_version": schema["schema_version"],
-                    "task_id": task["task_id"],
+                    "task_id": task_id,
                     "timestamp": datetime.utcnow().isoformat() + "Z",
                     "previous_data": previous_data
                 })
                 db[output_collection].delete_many({
-                    "pipeline_id": schema["pipeline_id"],
+                    "pipeline_id": pipeline_id,
                     "schema_version": schema["schema_version"],
-                    "task_id": task["task_id"]
+                    "task_id": task_id
                 })
                 db[output_collection].insert_one(updated_result)
                 return updated_result
@@ -247,30 +263,30 @@ def execute_task(task: Dict[str, Any], schema: Dict[str, Any], db: Any, sub_sche
                     if isinstance(item, dict):
                         updated_item = item.copy()
                         updated_item.update({
-                            "pipeline_id": schema["pipeline_id"],
+                            "pipeline_id": pipeline_id,
                             "schema_version": schema["schema_version"],
-                            "task_id": task["task_id"],
+                            "task_id": task_id,
                             "timestamp": datetime.utcnow().isoformat() + "Z",
                             "previous_data": previous_data if not updated_results else {}
                         })
                         db[output_collection].delete_many({
-                            "pipeline_id": schema["pipeline_id"],
+                            "pipeline_id": pipeline_id,
                             "schema_version": schema["schema_version"],
-                            "task_id": task["task_id"],
+                            "task_id": task_id,
                             "granularity": updated_item.get("granularity")
                         })
                         db[output_collection].insert_one(updated_item)
                         updated_results.append(updated_item)
                     else:
-                        logger.warning(f"Skipping non-dictionary item in list for task {task['task_id']}")
+                        logger.warning(f"Skipping non-dictionary item in list for task {task_id} in pipeline {pipeline_id}")
                 return updated_results
         else:
             if isinstance(result, dict):
                 updated_result = result.copy()
                 updated_result.update({
-                    "pipeline_id": schema["pipeline_id"],
+                    "pipeline_id": pipeline_id,
                     "schema_version": schema["schema_version"],
-                    "task_id": task["task_id"],
+                    "task_id": task_id,
                     "timestamp": datetime.utcnow().isoformat() + "Z"
                 })
                 db[output_collection].insert_one(updated_result)
@@ -281,32 +297,33 @@ def execute_task(task: Dict[str, Any], schema: Dict[str, Any], db: Any, sub_sche
                     if isinstance(item, dict):
                         updated_item = item.copy()
                         updated_item.update({
-                            "pipeline_id": schema["pipeline_id"],
+                            "pipeline_id": pipeline_id,
                             "schema_version": schema["schema_version"],
-                            "task_id": task["task_id"],
+                            "task_id": task_id,
                             "timestamp": datetime.utcnow().isoformat() + "Z"
                         })
                         db[output_collection].insert_one(updated_item)
                         updated_results.append(updated_item)
                     else:
-                        logger.warning(f"Skipping non-dictionary item in list for task {task['task_id']}")
+                        logger.warning(f"Skipping non-dictionary item in list for task {task_id} in pipeline {pipeline_id}")
                 return updated_results
 
-        raise ValueError(f"Unsupported result type {type(result)} for task {task['task_id']}")
+        raise ValueError(f"Unsupported result type {type(result)} for task {task_id} in pipeline {pipeline_id}")
     except Exception as e:
-        logger.error(f"Task {task['task_id']} execution failed: {str(e)}")
-        return {"status": "failed", "error": str(e), "task_id": task["task_id"]}
+        logger.error(f"Task {task_id} in pipeline {pipeline_id} execution failed: {str(e)}")
+        return {"status": "failed", "error": str(e), "task_id": task_id}
 
 def execute_pipeline(client: Any, db_name: str, schema: Dict[str, Any]) -> None:
     """Execute the entire pipeline with parallel task execution, including sub-schemas."""
-    logger.info(f"Starting pipeline execution for {schema['pipeline_id']}")
+    pipeline_id = schema.get("pipeline_id", "unknown")
+    logger.info(f"Starting pipeline execution for {pipeline_id}")
     db = client[db_name]
 
     # Validate schema
     try:
         validate_schema(schema)
     except ValueError as e:
-        logger.error(f"Schema validation failed: {str(e)}")
+        logger.error(f"Schema validation failed for pipeline {pipeline_id}: {str(e)}")
         raise
 
     # Load sub-schemas
@@ -315,15 +332,15 @@ def execute_pipeline(client: Any, db_name: str, schema: Dict[str, Any]) -> None:
         try:
             sub_schema_path = Path(sub_schema_path)
             if not sub_schema_path.is_file():
-                logger.error(f"Sub-schema file not found: {sub_schema_path}")
+                logger.error(f"Sub-schema file not found for pipeline {pipeline_id}: {sub_schema_path}")
                 continue
             with sub_schema_path.open("r") as f:
                 sub_schema = json.load(f)
             validate_schema(sub_schema)
             sub_schemas.append(sub_schema)
-            logger.info(f"Loaded sub-schema: {sub_schema['pipeline_id']}")
+            logger.info(f"Loaded sub-schema {sub_schema['pipeline_id']} for pipeline {pipeline_id}")
         except Exception as e:
-            logger.error(f"Failed to load sub-schema {sub_schema_path}: {str(e)}")
+            logger.error(f"Failed to load sub-schema {sub_schema_path} for pipeline {pipeline_id}: {str(e)}")
             continue
 
     # Build DAG for main schema
@@ -346,12 +363,12 @@ def execute_pipeline(client: Any, db_name: str, schema: Dict[str, Any]) -> None:
         resolved_schema["tasks"].append(task_copy)
 
     # Save resolved schema
-    schema_path = Path(f"schemas/{schema['pipeline_id']}.json")
-    output_dir = schema_path.parent / "resolved" / f"{schema['pipeline_id']}_{schema['schema_version']}.json"
+    schema_path = Path(f"schemas/{pipeline_id}.json")
+    output_dir = schema_path.parent / "resolved" / f"{pipeline_id}_{schema['schema_version']}.json"
     output_dir.parent.mkdir(parents=True, exist_ok=True)
     with output_dir.open("w") as f:
         json.dump(resolved_schema, f, indent=2)
-    logger.info(f"Saved resolved schema to {output_dir}")
+    logger.info(f"Saved resolved schema to {output_dir} for pipeline {pipeline_id}")
 
     # Execute tasks in parallel
     with concurrent.futures.ThreadPoolExecutor() as executor:
@@ -363,28 +380,29 @@ def execute_pipeline(client: Any, db_name: str, schema: Dict[str, Any]) -> None:
 
         for future in concurrent.futures.as_completed(future_to_task):
             task = future_to_task[future]
+            task_id = task.get("task_id", "unknown")
             try:
-                context[task["task_id"]] = future.result()
-                logger.info(f"Task {task['task_id']} executed successfully")
-                for successor in dag.successors(task["task_id"]):
+                context[task_id] = future.result()
+                logger.info(f"Task {task_id} in pipeline {pipeline_id} executed successfully")
+                for successor in dag.successors(task_id):
                     successor_task = next(t for t in tasks if t["task_id"] == successor)
                     if all(pred in context for pred in dag.predecessors(successor)):
                         future = executor.submit(execute_task, successor_task, schema, db, sub_schemas)
                         future_to_task[future] = successor_task
             except Exception as e:
-                logger.error(f"Task {task['task_id']} failed: {str(e)}")
+                logger.error(f"Task {task_id} in pipeline {pipeline_id} failed: {str(e)}")
                 if not schema["global_config"]["retry_on_failure"]:
                     raise
                 for attempt in range(schema["global_config"]["max_retries"]):
                     try:
-                        context[task["task_id"]] = execute_task(task, schema, db, sub_schemas)
-                        logger.info(f"Task {task['task_id']} retry succeeded")
+                        context[task_id] = execute_task(task, schema, db, sub_schemas)
+                        logger.info(f"Task {task_id} in pipeline {pipeline_id} retry succeeded")
                         break
                     except Exception as retry_e:
-                        logger.error(f"Retry {attempt + 1} failed for task {task['task_id']}: {str(retry_e)}")
+                        logger.error(f"Retry {attempt + 1} failed for task {task_id} in pipeline {pipeline_id}: {str(retry_e)}")
                         time.sleep(2)
                 else:
-                    raise Exception(f"Task {task['task_id']} failed after {schema['global_config']['max_retries']} retries")
+                    raise Exception(f"Task {task_id} in pipeline {pipeline_id} failed after {schema['global_config']['max_retries']} retries")
 
     # Execute sub-schemas in parallel
     with concurrent.futures.ThreadPoolExecutor() as executor:
@@ -398,15 +416,17 @@ def execute_pipeline(client: Any, db_name: str, schema: Dict[str, Any]) -> None:
             sub_schema = sub_schema_futures[future]
             try:
                 future.result()
-                logger.info(f"Sub-schema {sub_schema['pipeline_id']} executed successfully")
+                logger.info(f"Sub-schema {sub_schema['pipeline_id']} executed successfully for pipeline {pipeline_id}")
             except Exception as e:
-                logger.error(f"Sub-schema {sub_schema['pipeline_id']} failed: {str(e)}")
+                logger.error(f"Sub-schema {sub_schema['pipeline_id']} failed for pipeline {pipeline_id}: {str(e)}")
                 # Continue with other sub-schemas instead of failing the entire pipeline
 
-    logger.info(f"Pipeline {schema['pipeline_id']} execution completed")
+    logger.info(f"Pipeline {pipeline_id} execution completed")
 
 def execute_single_task(client: Any, db_name: str, schema: Dict[str, Any], task: Dict[str, Any]) -> Dict[str, Any]:
     """Execute a single task via API request."""
+    pipeline_id = schema.get("pipeline_id", "unknown")
+    task_id = task.get("task_id", "unknown")
     db = client[db_name]
     # Load sub-schemas for input resolution
     sub_schemas = []
@@ -415,8 +435,9 @@ def execute_single_task(client: Any, db_name: str, schema: Dict[str, Any], task:
             with Path(sub_schema_path).open("r") as f:
                 sub_schema = json.load(f)
             sub_schemas.append(sub_schema)
+            logger.info(f"Loaded sub-schema {sub_schema['pipeline_id']} for task {task_id} in pipeline {pipeline_id}")
         except Exception as e:
-            logger.error(f"Failed to load sub-schema {sub_schema_path}: {str(e)}")
+            logger.error(f"Failed to load sub-schema {sub_schema_path} for task {task_id} in pipeline {pipeline_id}: {str(e)}")
     return execute_task(task, schema, db, sub_schemas)
 
 if __name__ == "__main__":
